#!/usr/bin/ruby

require "Citrusleaf"
c = Citrusleaf.new
puts "connecting to cluster!"

host="127.0.0.1"
port=3000
c.add_node host, port
puts "connected to %s:%d" % [host,port]

require "set"

ns = 'test'
set = 'rubytestset'

#
# create a little hash of strings and values to set
#
values_out = { }
x = 10
while x >= 0 do
   values_out[x.to_s] = x.to_s * 3
   x -= 1
end

puts "*** test: setting hash of 10 values, getting 2 back, and one that doesn't exist"

c.delete(ns,set,'rubytestkey')
c.put ns, set, 'rubytestkey', values_out

rv, gen, vals = c.get ns, set, 'rubytestkey', [ 'been', '6', '7' ]
if rv != 0
   puts "error from put: #{r}"
   Process.exit
end

#
# When running against a single-bin server, the effective value of the key is the last value in the put list.
# This value is returned as the the value of the first bin, regardless of the bin-name given.
# With a single bin environment, continue running the example is just confusing and also will crash, 
# so it is better to quit out early
#
if (vals['been'] == '000')
   puts 'The Aerospike server is not multi-bin enabled. Please run this test against a multi-bin enabled server'
   Process.exit
end

puts "Print returned values #{vals}"
vals.each do |key,value|
   puts "%s %s #{value.class} " % [key,value]
   if value.class == "".class
      puts "value is string, length #{value.length}"
   end
end

sixval = vals['6']
if ( sixval != "666" )
   puts "ERROR: 6 should be 666 is #{sixval}."
   sixval.each_byte { |x|
      puts x
   }
   Process.exit
end

puts "test: ruby uses 'string' very liberally, support strings with null bytes"
badstr = ""
(1..513).each { |x| badstr.concat(x.to_s) }

c.delete ns, set, 'ruby3'
c.put ns, set, 'ruby3', badstr:badstr

rv, gen, badstr2 = c.get ns, set, 'ruby3', 'badstr'
if rv != 0
   puts "error: badstr couldn't get value back rv ", rv
   Process.exit
end
if badstr2.class != "".class
   puts "error: badstr2 should have string class, instead is #{badstr2.class}"
   Process.exit
end
(0..512).each { |x|
   if badstr[x] != badstr2[x]
      puts "badstr2 differs from badstr at position x, %d should be %d" % [badstr2[x],badstr[x]]
   end
}
puts " ... strings with nulls inside work fine"

puts "test: using number as a simple value"

c.delete ns, set, 'rubytestkey2'

c.put ns, set, 'rubytestkey2', "5"=>5
c.put ns, set, 'rubytestkey2', "6"=>6

rv, gen, vals = c.get ns, set, 'rubytestkey2'

if rv != 0
   puts "could not retrieve value rv #{rv}"
   Process.exit
end

vals.each do |key,value|
   puts "value class: #{value.class}"
   puts "#{key} value #{value}"
end

if vals.length != 2
   puts "should have retrieved just two values, got #{vals.length} instead"
   Process.exit
end

if vals["5"] != 5
   puts " ERROR: five should be five, is #{vals["5"]}"
   Process.exit
end
if vals["6"] != 6
   puts " ERROR: six should be six, is #{vals["6"]}"
   Process.exit
end

rv, gen, val = c.get ns, set, 'rubytestkey2', "5"

puts "rv is #{rv}"

if val != 5
   puts " five should be five, is #{val}"
   Process.exit
end

puts "integer test success"

puts "starting test of non existent keys"

c.delete ns, set, "ruby555"
rv, gen, vals = c.get ns, set, "ruby555"
if rv != 2
   puts " key doesn't exist, rv returned #{rv}, should have returned 2"
   Process.exit
end

puts "test of non existant key succeeded"

puts "test of Marshalled data"

# create a complicated data structure
s = Set.new
(1..6).each { |x| s.add(x) }
s.add "brian"
s.add "bulkowski"
s.add 4.23
mar = Marshal.dump s

puts "created a marshalled string length #{mar.length}"

c.delete ns, set, "rubym"
c.put ns, set, "rubym", "default", mar

rv, gen, mar2 = c.get ns, set, "rubym", "default"
if rv != 0
   puts "problem fetching marshalled data rv #{rv}"
   Process.exit
end

puts "fetched string length #{mar2.length}"
s2 = Marshal.load mar2
puts " class after demarshalling: #{s2.class}"
s2.each { |x| puts x }
if s.length != 9
   puts "length of set is wrong, should be 9 is #{s2.length}"
   Process.exit
end

puts "test of auto-marshalling"

c.delete ns, set, "rubym2"
c.put ns, set, "rubym2", "default", s

rv, gen, mar2 = c.get ns, set, "rubym2", "default"
if rv != 0
   puts "problem fetching automashalled data rv #{rv}"
   Process.exit
end

if mar2.class != s.class
   puts "FAIL: auto-marshal fail: expected class #{s.class} found #{mar2.class}"
   Process.exit
end

mar2.each { |x| puts x }
if mar2.length != 9
   puts "length of set is wrong, should be 9 is #{mar2.length}"
   Process.exit
end

puts "auto-marshall success"



